
use DBI;
use PFM::remove_grammar;

# Connect to the database.
my $dsn = 'DBI:mysql:lhai:localhost';
my $db_user_name = 'inet';
my $db_password = '';
my $dbh = DBI->connect($dsn, $db_user_name, $db_password);

# set $no_censor to 1 if you wish to hear altbot say censored words.
my $no_censor = 0;

if(!$dbh) {
	die "Can\'t find database!";
}

sub exit_ai
{
	# Clean up the connection.
	$dbh->disconnect();
}

# This returns a list of users that match the username given.
sub get_user
{
	my $user = shift;
	
	# TODO: Make sure that SQL injection is not possible.
	$user =~ s/"/\\"/g;
	$user =~ s/\\/\\\\/g;
	
	my $sth = $dbh->prepare(qq{
	select * from member 
		where username = "$user"
	});
	$sth->execute();
	
	my (@matrix) = ();
	while (my @ary = $sth->fetchrow_array())
	{
		push(@matrix, [@ary]);  # [@ary] is a reference.
	}
	$sth->finish();
	
	foreach $val (@matrix) {
		# @$val dereferences @ary.
		my @data = @$val;
		# quite annoying that email field is commonly null.
		# produces a warning if I don't do this. All field should be 
		# NOT NULL default '' for this reason alone.
		if(!defined($data[2])) { $data[2] = "(null)"; }
		#print "User info: @data\n";
	}
	# return the list of users.
	@matrix;
}

# This adds a user with the given username.
sub add_user
{
	my $user = shift;
	
	# TODO: Make sure that SQL injection is not possible.
	$user =~ s/\\/\\\\/g;
	$user =~ s/"/\\"/g;
	
	my $sth = $dbh->prepare(qq{
	insert into member VALUES ("unknown", "$user", "")
	});
	$sth->execute();
	# return 1.
	1;
}

# This adds a record with the given username, data, and date.
sub input_data
{
	my $user = shift;
	my $body = shift;
	my $date = shift;

	# TODO: Make sure that SQL injection is not possible.
	chomp($body);
	# strip quotes or add slashes. That ought to keep out the sql injections.
	$body =~ s/\\/\\\\/g;
	$user =~ s/\\/\\\\/g;
	$body =~ s/"/\\"/g;
	$user =~ s/"/\\"/g;
	
	my $sth = $dbh->prepare(qq{
	insert into data VALUES (0, "$user", "$body")
	});
	$sth->execute();
	# return 1.
	1;
}

# This searches for data that is similar to the given data.
sub find_data
{
	my $body = shift;
	my $use_rand = shift || 1;
	
	my $keyword = &get_keyword($body);
	
	my $order = "id desc";
	if($use_rand)
	{
		$order = "rand()";
	}
	
	# strip quotes or add slashes. That ought to keep out the sql injections.
	$keyword =~ s/\\/\\\\/g;
	$keyword =~ s/"/\\"/g;
	
	# TODO: Make sure that SQL injection is not possible.
	my $sth = $dbh->prepare(qq{
	select * from data
		where body like "%$keyword%"
		order by $order
		limit 1
	});
	$sth->execute();
	
	my (@matrix) = ();
	while (my @ary = $sth->fetchrow_array())
	{
		push(@matrix, [@ary]);  # [@ary] is a reference.
	}
	$sth->finish();
	
	my $found = "";
	foreach $val (@matrix) {
		# @$val dereferences @ary.
		my @data = @$val;
		#print "Data: @data\n";
		# if there are no bad words, say it.
		if($no_censor || &get_bad_word($data[2]) == 0) {
			$found .= $data[2];
		}
	}
	# return the list of data.
	#@matrix;
	$found;
}

# each bad word can contain regex, which is useful.
@bad_words1 = ('New post :: '); #, 'f?.k', 'c..t');

# returns true if a bad word as defined above is said.
sub get_bad_word
{
	my ($input) = @_;
	my $found = 0;
	foreach $search (@bad_words1) {
		if($input =~ /$search/g) {
			# we found the bad word, return.
			$found = 1;
			last;
		}
		else {
			# no bad word found, keep searching.
			$found = 0;
		}
	}
	# return true or false, depending on found.
	$found;
}

# This finds the most useful word or phrase in the sentence.
# This is where the development should go.
# It picks only nouns, adjectives, verbs, or adverbs.
sub get_keyword
{
	my $body = shift;
	# If there's a \n at the end, all hell breaks loose.
	chomp($body);
	#print "$body\n";
	# get rid of all words that are well known and unuseful.
	# get rid all words well known unuseful.
	$body = remove_grammar($body);
	my $keyword = $body;
	my @words = split / /, $body;
	# get a random word from the useful words.
	$index = int(rand $#words);
	$keyword = $words[$index];
	# return keyword.
	$keyword;
}

# This finds the most useful word or phrase in the sentence.
# Currently it selects the longest word.
# We could use a weighting system if we wanted.
sub get_longest_keyword
{
	my $body = shift;
	# If there's a \n at the end, all hell breaks loose.
	chomp($body);
	#print "$body\n";
	my $keyword = $body;
	my @words = split / /, $body;
	my $max_len = 0;
	foreach $word (@words) {
		# Longest word wins.
		$len = length $word;
		if($len > $max_len) {
			#print "$word $len > $max_len\n";
			$max_len = $len;
			$keyword = $word;
		}
	}
	# return keyword.
	$keyword;
}

1;
